import json
import logging
from django.http import HttpResponse, JsonResponse
from django.template import loader

from ..plugins import Plugins


# Module logger
mogger = logging.getLogger("pza.interfaces.io")



###############################################################################
###############################################################################

def ui_multi_control(request):
    """
    """
    template = loader.get_template('io_multi_control.html')
    
    io_interfaces = []

    adapters = Plugins.GetAdapters()
    for a in adapters:
        for i in adapters[a]:
            if adapters[a][i]["type"] == "io":
                io_interfaces.append(str(a) + "/" + str(i))

    return HttpResponse(template.render(
        {
            "io_interfaces": io_interfaces
        },
        request=request))



###############################################################################
###############################################################################

def ui_single_control(request, adapter, interface):
    """
    """
    template = loader.get_template('io_single_control.html')
    
    return HttpResponse(template.render(
        {
            "target_interface": str(adapter) + "/" + str(interface),
            "plugin_status": Plugins.GetStatus()
        },
        request=request))

###############################################################################
###############################################################################

def value(request, adapter, interface):
    """
    """
    # Debug
    mogger.debug("[%s] io.value %s %s", request.method, adapter, interface)
    
    try:
        # Find plugin
        plugin = Plugins.FindPluginFromAdapter(adapter_name=adapter)
        mogger.debug("Plugin found for adapter [%s] => [%s] ", adapter, plugin.Name )

        # Manage the GET method
        if request.method == "GET":
            mogger.debug("GET method")
            return JsonResponse( { 'value': plugin.IoValueRead(adapter, interface) } )

        # Manage the PUT method
        elif request.method == "PUT":
            body_unicode = request.body.decode('utf-8')
            mogger.debug("PUT method %s", body_unicode)
            body = json.loads(body_unicode)
            plugin.IoValueWrite(adapter, interface, body["value"])
            return HttpResponse(status=200)

        # Others are errors
        else:
            mogger.warning("Unknown method %s", request.method)
            return HttpResponse(status=405)

    except Exception as e:
        mogger.error(repr(e))

###############################################################################
###############################################################################

def direction(request, adapter, interface):
    """
    """
    # Debug
    mogger.debug("[%s] io.direction %s %s", request.method, adapter, interface)
    
    try:
        # Find plugin
        plugin = Plugins.FindPluginFromAdapter(adapter_name=adapter)
        mogger.debug("Plugin found for adapter [%s] => [%s] ", adapter, plugin.Name )
        
        # Manage the GET method
        if request.method == "GET":
            mogger.debug("GET method")
            return JsonResponse( { "direction": plugin.IoDirectionRead(adapter, interface) } )

        # Manage the PUT method
        elif request.method == "PUT":
            body_unicode = request.body.decode('utf-8')
            mogger.debug("PUT method %s", body_unicode)
            body = json.loads(body_unicode)
            plugin.IoDirectionWrite(adapter, interface, body["direction"])
            return HttpResponse(status=200)

        # Others are errors
        else:
            mogger.warning("Unknown method %s", request.method)
            return HttpResponse(status=405)

    except Exception as e:
        mogger.error(repr(e))
        return HttpResponse(repr(e), status=500)

###############################################################################
###############################################################################

def active_low(request, adapter, interface):
    # print(">>>", request, request.method)

    return JsonResponse( Plugins.GetInterfaces() )






"""panduza URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.urls import path


from panduza.views import plugins, index, interfaces, interfaces_ui

from .interfaces import io


urlpatterns = [
    path('', index),
    path('ui/interfaces', interfaces_ui),
    # path('ui/plugins', plugins_ui),

    path('ui/io', io.ui_multi_control),
    path('ui/io/<str:adapter>/<str:interface>', io.ui_single_control),

    path('plugins', plugins),
    # path('plugin/<str:name>/conf', plugins),

    path('interfaces', interfaces),
    
    path('io/<str:adapter>/<str:interface>/value', io.value),
    path('io/<str:adapter>/<str:interface>/direction', io.direction),
    # path('io/<str:adapter>/<str:interface>/active_low', io.active_low),
]





from setuptools import setup, find_packages
import glob

VERSION = '0.0.1'
DESCRIPTION = 'Panduza Core Server'
LONG_DESCRIPTION = 'The Panduza server provide a bridge to control hardware interfaces'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="panduza_server", 
        version=VERSION,
        author="Panduza Team",
        author_email="panduza.team@gmail.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        
        install_requires=['django', 'djangorestframework', 'django-cors-headers'],

        package_data={
            'panduza_server': [
                'static/css/*.css',
                'static/img/*.png',
                'static/js/*.js',
                'templates/*.html'
            ]
        },

        entry_points = {
            'console_scripts': ['pza-run-server=panduza_server.scripts.server:run_server'],
        },
            
        keywords=['python', 'first package'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)





import os
import panduza_server
import subprocess

def run_server():
    server_dir = os.path.dirname(panduza_server.__file__)
    print("SERVER DIRECTORY: ", server_dir)

    subprocess.run(["python3" ,  server_dir + "/manage.py", "runserver"])



"""
Django settings for panduza project.

Generated by 'django-admin startproject' using Django 3.2.4.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

import os
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-6rwv#ke#yirvp52s(jchf$m-dpssh&ydh=nv9et=5c=+b$pq##'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

#
# ALLOWED_HOSTS = ['192.168.2.1']
# ALLOWED_HOSTS = ['192.168.1.39']
ALLOWED_HOSTS = ['*']

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'cli': {
            'format': '{levelname} - [{name}] {message}',
            'style': '{',
        },
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'cli'
        },
    },
    'loggers': {
        'pza': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'corsheaders',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    # 'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]


# CORS_ALLOWED_ORIGINS = [
#     "http://localhost:8000",
#     "http://127.0.0.1:8000"
# ]
CORS_ALLOW_ALL_ORIGINS = True


ROOT_URLCONF = 'panduza.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [ BASE_DIR / 'templates' ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'panduza.wsgi.application'


# Database
# https://docs.djangoproject.com/en/3.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/3.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.2/howto/static-files/

STATIC_URL = '/static/'

STATICFILES_DIRS = [ os.path.join(BASE_DIR,'static') ]

STATIC_ROOT = os.path.join(BASE_DIR, 'assets')

# Default primary key field type
# https://docs.djangoproject.com/en/3.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
